# Created by Q-ays.
# whosqays@gmail.com


err_num = {
    "OK": 0,
    "DATAERR": 1,
    "DSANF": 2,
    "PARM": 3,
    "PARGERR": 4,
    "NOadmin": 5,
    "NOAUTH": 6
}

err_msg = {
    "OK": "OK",
    "DATAERR": "数据库错误",
    "DSANF": "第三方服务错误",
    "PARM": "参数错误",
    "PARGERR": "数据错误",
    "NOadmin": "被管理者",
    "NOAUTH": "没有权限"
}


"""
    Example::

        from wisdoms.auth import permit

        host = {'AMQP_URI': "amqp://guest:guest@localhost"}

        auth = permit(host)

        class A:
            @auth
            def func():
                pass
"""

from nameko.standalone.rpc import ClusterRpcProxy
from nameko.rpc import rpc
from wisdoms.utils import exception
from functools import wraps


def permit(host):
    """ 调用微服务功能之前，进入基础微服务进行权限验证

    :param: host: micro service host
    """

    def wrapper(f):
        @wraps(f)
        def inner(*args, **kwargs):
            srv_name = args[0].name
            uid = args[1]['uid']
            f_name = f.__name__
            with ClusterRpcProxy(host) as rpc:
                res = rpc.db_service.isrole_app(srv_name, uid, f_name)
            if res['code'] == 1:
                return f(*args, **kwargs)
            return res

        return inner

    return wrapper


def ms_base(ms_host):
    """
    返回父类，闭包，传参数ms host
    :param ms_host:
    :return:
    """

    class MsBase:
        name = 'ms-base'

        @rpc
        @exception()
        def into_base(self):
            clazz = type(self)
            name = clazz.name
            functions = list(clazz.__dict__.keys())
            with ClusterRpcProxy(ms_host) as r:
                r.db_service.into_base(name, functions)

    return MsBase



'''
第一种方法：直接在模块中定义方法
```
import wisdom.example as we
we.hello()
```
'''
import logging
from wisdoms.commons import revert, codes, success
def hello():
    print('hello{}'.format('world'))

def world():
    print('{},world'.format('hello'))

if __name__ == '__main__':
    print(revert(codes.NOT_AUTHORIZED))
    print(success('abc'))
    print(success())


# Created by Q-ays.
# whosqays@gmail.com

from wisdoms.commons import revert, codes


def joint_base2(url1, url2):
    url1 = str(url1)
    url2 = str(url2)
    if url1.endswith('/') and not url2.startswith('/'):
        return url1 + url2
    elif not url1.endswith('/') and url2.startswith('/'):
        return url1 + url2
    elif url1.endswith('/') and url2.startswith('/'):
        return url1 + url2[1:]
    else:
        return url1 + '/' + url2


def joint4path(*args):
    """
    连接n个路径
    :param args:a,b,c,d
    :return: a/b/c/d
    """
    url1 = args[0]

    length1 = len(args)
    if length1 > 1:
        for i in range(1, length1):
            url1 = joint_base2(url1, args[i])

    return url1


def o2d(obj):
    """
    把对象(支持单个对象、list、set)转换成字典
    :param obj: obj, list, set
    :return:
    """
    is_list = isinstance(obj, list)
    is_set = isinstance(obj, set)

    if is_list or is_set:
        obj_arr = []
        for o in obj:
            # 把Object对象转换成Dict对象
            dict1 = {}
            dict1.update(o.__dict__)
            obj_arr.append(dict1)
        return obj_arr
    else:
        dict1 = {}
        dict1.update(obj.__dict__)
        return dict1


def exception(code=codes.ERROR):
    """
    捕获异常装饰器
    :param code:
    :return:
    """

    def wrapper(func):
        def catch(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                print(e)
                return revert(code, e)

        return catch

    return wrapper
